@@ -9,7 +9,7 @@
 # Modules and declarations
 ##############################################################################
 
-package App::DocKnot::Util 6.00;
+package App::DocKnot::Util 6.01;
 
 use 5.024;
 use autodie;
@@ -18,8 +18,9 @@ use warnings;
 use Carp qw(croak);
 use Exporter qw(import);
 use List::SomeUtils qw(all);
+use Sort::Versions qw(versioncmp);
 
-our @EXPORT_OK = qw(is_newer print_checked print_fh);
+our @EXPORT_OK = qw(is_newer latest_tarball print_checked print_fh);
 
 ##############################################################################
 # Public interface
@@ -39,6 +40,47 @@ sub is_newer {
     return all { $file_mtime >= $_ } @others_mtimes;
 }
 
+# Find the files for a given package with the latest version and return them
+# along with some associated metadata.
+#
+# $path    - Path::Tiny path to directory
+# $tarname - Name of the tarball before the version component
+#
+# Returns: Anonymous hash with the following keys:
+#            version - Latest version found
+#            date    - Date (in seconds since epoch) of oldest file
+#            files   - Array of files for that version
+#          or undef if no matching files were found
+#  Throws: Text exception on any error
+sub latest_tarball {
+    my ($path, $tarname) = @_;
+
+    # Collect the list of matching files and extract their version numbers.
+    return if !$path->is_dir();
+    my $regex = qr{ \A \Q$tarname\E - ([\d.]+) [.] }xms;
+    my @files = map { $_->basename() } $path->children($regex);
+    my @versions = map { m{ $regex }xms ? [$1, $_] : () } @files;
+    return if !@versions;
+
+    # Find the latest version and filter the list of files down to only that
+    # version.
+    @versions = reverse(sort { versioncmp($a->[0], $b->[0]) } @versions);
+    my $latest = $versions[0][0];
+    @files = map { $_->[1] } grep { $_->[0] eq $latest } @versions;
+
+    # Find the timestamps of those files.
+    my @times = sort(map { $path->child($_)->stat()->[9] } @files);
+
+    # Return the results.
+    #<<<
+    return {
+        version => $latest,
+        date    => $times[0],
+        files   => \@files,
+    };
+    #<<<
+}
+
 # print with error checking.  autodie unfortunately can't help us because
 # print can't be prototyped and hence can't be overridden.
 #
@@ -95,7 +137,8 @@ App::DocKnot::Util - Shared utility functions for other DocKnot modules
 
 =head1 REQUIREMENTS
 
-Perl 5.24 or later and the List::SomeUtils module, available from CPAN.
+Perl 5.24 or later and the modules List::SomeUtils and Sort::Versions,
+available from CPAN.
 
 =head1 DESCRIPTION
 
@@ -114,6 +157,28 @@ or equal to the last modified times of all SOURCE files, and otherwise returns
 a false value.  Used primarily to determine if a given output file is
 up-to-date with respect to its source files.
 
+=item latest_tarball(PATH, NAME)
+
+Returns data including a file list for the latest tarballs (by version number)
+for a given software package NAME in the directory PATH.  Versions are compared
+using Sort::Versions.  The return valid is a hash with the following keys:
+
+=over 4
+
+=item date
+
+The timestamp of the oldest file for that version, in seconds since epoch.
+
+=item files
+
+The list of files found for that version.
+
+=item version
+
+The version number extracted from this set of files.
+
+=back
+
 =item print_checked(ARG[, ARG ...])
 
 The same as print (without a file handle argument), except that it throws a
@@ -123,8 +188,8 @@ doesn't because print cannot be prototyped).
 =item print_fh(FH, NAME, DATA[, DATA ...])
 
 Writes the concatenation of the DATA elements (interpreted as scalar strings)
-to the file handle FH.  NAME should be the name of the file open as FH, and is
-used for error reporting.
+to the file handle FH.  NAME should be the name of (or Path::Tiny object for)
+the file open as FH, and is used for error reporting.
 
 This is mostly equivalent to C<print {fh}> but throws a text exception in the
 event of a failure.
@@ -137,7 +202,7 @@ Russ Allbery <rra@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 1999-2011, 2013, 2021 Russ Allbery <rra@cpan.org>
+Copyright 1999-2011, 2013, 2021-2022 Russ Allbery <rra@cpan.org>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
